
def factorial(n:int) -> float:
    resultado = 1
    for i in range(2, n + 1):
        resultado *= i
    return resultado

#IEC61508
def Tce( λ_du: float,λ_dd: float,T1_month: float,MTTR: float) -> float:
    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    val = (λ_du/λ_d)*((T1_hour/2)+MTTR)+(λ_dd/λ_d)*MTTR
    return val

def Tce_1pt(λ_du: float,λ_dd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:

    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    T_pt_hour = T_pt1_month*730
    val = (λ_du*(1-PDC)/λ_d)*((T1_hour/2)+MTTR)+(λ_du*(PDC)/λ_d)*((T_pt_hour/2)+MTTR)+(λ_dd/λ_d)*MTTR
    return val

#IEC61508
def Tge(λ_du: float,λ_dd: float,T1_month: float,MTTR: float) -> float:
    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    val = (λ_du/λ_d)*((T1_hour/3)+MTTR)+λ_dd/λ_d*MTTR
    return val

def Tge_1pt(λ_du: float,λ_dd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:
    
    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    T_pt_hour = T_pt1_month*730
    val = (λ_du*(1-PDC)/λ_d)*((T1_hour/3)+MTTR)+(λ_du*(PDC)/λ_d)*((T_pt_hour/3)+MTTR)+(λ_dd/λ_d)*MTTR
    return val

#IEC61508
def Tg2e(λ_du: float,λ_dd: float,T1_month: float,MTTR: float) -> float:
    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    val = (λ_du/λ_d)*((T1_hour/4)+MTTR)+λ_dd/λ_d*MTTR
    return val

def Tg2e_1pt(λ_du: float,λ_dd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:
    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    T_pt_hour = T_pt1_month*730
    val = (λ_du*(1-PDC)/λ_d)*((T1_hour/4)+MTTR)+(λ_du*(PDC)/λ_d)*((T_pt_hour/4)+MTTR)+(λ_dd/λ_d)*MTTR
    return val

def Tne(λ_du: float,λ_dd: float,n: int,T1_month: float,MTTR: float) -> float:
    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    val = 1 
    for i in range(1,n+1):
       val *= (λ_du/λ_d)*((T1_hour/(i+1))+MTTR)+λ_dd/λ_d*MTTR
    return val

def Tne_1pt(λ_du: float,λ_dd: float,n: int,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:
    λ_d =  λ_du + λ_dd
    T1_hour = T1_month*730
    T_pt_hour = T_pt1_month*730
    val = 1 
    for i in range(1,n+1):
       val *= (λ_du*(1-PDC)/λ_d)*((T1_hour/(i+1))+MTTR)+(λ_du*(PDC)/λ_d)*((T_pt_hour/(i+1))+MTTR)+(λ_dd/λ_d)*MTTR
    return val

def pfd_RBD_avg_1oo1(λ_du: float,λ_dd: float,T1_month: float,MTTR: float) -> float:
    val = (λ_du+λ_dd)*Tne(λ_du,λ_dd,1,T1_month,MTTR)
    return  val

def pfd_RBD_avg_1oo1_pt(λ_du: float,λ_dd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:
    val = (λ_du+λ_dd)*Tce_1pt(λ_du,λ_dd,T1_month,T_pt1_month,PDC,MTTR)
    return  val

def pfd_RBD_avg_1oo2(λ_du: float,λ_dd: float,β: float,βd: float,T1_month: float,MTTR: float) -> float:
    T1_hour = T1_month*730
    val = 2 * (((1-βd)*λ_dd + (1- β)*λ_du)**2)*Tne(λ_du,λ_dd,2,T1_month,MTTR) + βd*λ_dd*MTTR + β*λ_du*(T1_hour/2+MTTR)
    return   val

def pfd_RBD_avg_1oo2_pt(λ_du: float,λ_dd: float,β: float,βd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:
    T1_hour = T1_month*730
    T_pt_hour = T_pt1_month*730
    
    val = 2 * (((1-βd)*λ_dd + (1- β)*λ_du)**2)*Tce_1pt(λ_du,λ_dd,T1_month,T_pt1_month,PDC,MTTR)*Tge_1pt(λ_du,λ_dd,T1_month,T_pt1_month,PDC,MTTR) + \
          βd*λ_dd*MTTR + β*λ_du*(1-PDC)*(T1_hour/2+MTTR)+β*λ_du*(PDC)*(T_pt_hour/2+MTTR)
    
    return   val

def pfd_RBD_avg_1oo3(λ_du: float,λ_dd: float,β: float,βd: float,T1_month: float,MTTR: float) -> float:
    T1_hour = T1_month*730
    val = 6 * (((1-βd)*λ_dd + (1- β)*λ_du)**3)*Tne(λ_du,λ_dd,3,T1_month,MTTR) + βd*λ_dd*MTTR + β*λ_du*(T1_hour/2+MTTR)
    return   val

def pfd_RBD_avg_2oo2(λ_du: float,λ_dd: float,T1_month: float,MTTR: float) -> float:
    return  pfd_RBD_avg_1oo1(λ_du,λ_dd,T1_month,MTTR)*2

def pfd_RBD_avg_3oo3(λ_du: float,λ_dd: float,T1_month: float,MTTR: float) -> float:
    return  pfd_RBD_avg_1oo1(λ_du,λ_dd,T1_month,MTTR)*3

def pfd_RBD_avg_2oo2_pt(λ_du: float,λ_dd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:
    return  pfd_RBD_avg_1oo1_pt(λ_du,λ_dd,T1_month,T_pt1_month,PDC,MTTR)*2

def pfd_RBD_avg_3oo3_pt(λ_du: float,λ_dd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:
    return  pfd_RBD_avg_1oo1_pt(λ_du,λ_dd,T1_month,T_pt1_month,PDC,MTTR)*3

def pfd_RBD_avg_2oo3(λ_du: float,λ_dd: float,β: float,βd: float,T1_month: float,MTTR: float) -> float:
    T1_hour = T1_month*730
    val = 6 * (((1-βd)*λ_dd + (1- β)*λ_du)**2)*Tne(λ_du,λ_dd,2,T1_month,MTTR) + βd*λ_dd*MTTR + β*λ_du*(T1_hour/2+MTTR)
    return   val

def pfd_RBD_avg_KooN(K: int,N: int,λ_du: float,λ_dd: float,β: float,βd: float,T1_month: float,MTTR: float) -> float:

    T1_hour = T1_month*730
    
    if K == N:
       val = (λ_du+λ_dd)*Tne(λ_du,λ_dd,1,T1_month,MTTR) * K
    else:
       fator = N - K +1
       val = factorial(N) * (((1-βd)*λ_dd + (1- β)*λ_du)**(fator))*Tne(λ_du,λ_dd,fator,T1_month,MTTR) + βd*λ_dd*MTTR + β*λ_du*(T1_hour/2+MTTR)
    return   val

def pfd_RBD_avg_KooN_1pt(K: int,N: int,λ_du: float,λ_dd: float,β: float,βd: float,T1_month: float,T_pt1_month: float,PDC: float,MTTR: float) -> float:

    T1_hour = T1_month*730
    T_pt_hour = T_pt1_month *730
    if K == N:
       val = (λ_du+λ_dd)*Tne_1pt(λ_du,λ_dd,1,T1_month,T_pt1_month,PDC,MTTR) * K
    else:
       fator = N - K +1
       val = factorial(N) * (((1-βd)*λ_dd + (1- β)*λ_du)**(fator))*Tne_1pt(λ_du,λ_dd,fator,T1_month,T_pt1_month,PDC,MTTR) + βd*λ_dd*MTTR + β*λ_du*(1.0-PDC)*(T1_hour/2+MTTR) + β*λ_du*(PDC)*(T_pt_hour/2+MTTR)
    return   val


#in development
def pfh_RBD_KooN(K:int,N:int,λ_d: float,β: float,T1_month: float) -> float:
    T1_hour = T1_month*730
    val = factorial(N)/(factorial(N-K)*factorial(K-1)) * (1-β) * λ_d * ((1-β)*λ_d*T1_hour/2)**(N-K) + (β*λ_d)
    
    return   val

#in development
def pfd_RBD_avg_1oo2_dif(λ_du1: float,λ_dd1: float,T1_month1: float,MTTR1: float,β1: float,βd1: float,λ_du2: float,λ_dd2: float,T1_month2: float,MTTR2: float,β2: float,βd2: float) -> float:

    λ_d1 = λ_du1 + λ_dd1
    T1_hour1 = T1_month1*730
    λ_d2 = λ_du2 + λ_dd2
    T1_hour2 = T1_month2*730

    Tce1 = (λ_du1/λ_d1)*((T1_hour1/2)+MTTR1)+(λ_dd1/λ_d1)*MTTR1
    Tce2 = (λ_du2/λ_d2)*((T1_hour2/2)+MTTR2)+(λ_dd2/λ_d2)*MTTR2   

    Tge1 = (λ_du1/λ_d1)*((T1_hour1/3)+MTTR1)+(λ_dd1/λ_d1)*MTTR1
    Tge2 = (λ_du2/λ_d2)*((T1_hour2/3)+MTTR2)+(λ_dd2/λ_d2)*MTTR2   
    
    val = (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2))*Tce1*Tge2 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2))*Tce2*Tge1 + \
          (βd1+βd2)/2*(λ_dd1+λ_dd2)/2*(MTTR1+MTTR2)/2 + \
          (β1+β2)/2*(λ_du1+λ_du2)/2*((T1_hour1+T1_hour2)/2/2+(MTTR1+MTTR2)/2)

    return   val


def pfd_RBD_avg_2oo3_dif(λ_du1: float,λ_dd1: float,T1_month1: float,MTTR1: float,β1: float,βd1: float,λ_du2: float,λ_dd2: float,T1_month2: float,MTTR2: float,β2: float,βd2: float,λ_du3: float,λ_dd3: float,T1_month3: float,MTTR3: float,β3: float,βd3: float) -> float:

    λ_d1 = λ_du1 + λ_dd1
    T1_hour1 = T1_month1*730

    λ_d2 = λ_du2 + λ_dd2
    T1_hour2 = T1_month2*730

    λ_d3 = λ_du3 + λ_dd3
    T1_hour3 = T1_month3*730


    Tce1 = (λ_du1/λ_d1)*((T1_hour1/2)+MTTR1)+(λ_dd1/λ_d1)*MTTR1
    Tce2 = (λ_du2/λ_d2)*((T1_hour2/2)+MTTR2)+(λ_dd2/λ_d2)*MTTR2
    Tce3 = (λ_du3/λ_d3)*((T1_hour3/2)+MTTR3)+(λ_dd3/λ_d3)*MTTR3 
   
    Tge1 = (λ_du1/λ_d1)*((T1_hour1/3)+MTTR1)+(λ_dd1/λ_d1)*MTTR1
    Tge2 = (λ_du2/λ_d2)*((T1_hour2/3)+MTTR2)+(λ_dd2/λ_d2)*MTTR2   
    Tge3 = (λ_du3/λ_d3)*((T1_hour3/3)+MTTR3)+(λ_dd3/λ_d3)*MTTR3

    val = (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2))*Tce1*Tge2 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce1*Tge3 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2))*Tce2*Tge1 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce3*Tge1 + \
          (((1-βd3)*λ_dd3 + (1- β3)*λ_du3)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2))*Tce2*Tge3 + \
          (((1-βd3)*λ_dd3 + (1- β3)*λ_du3)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2))*Tce3*Tge2 + \
          (βd1+βd2+βd3)/3*(λ_dd1+λ_dd2+λ_dd3)/3*(MTTR1+MTTR2+MTTR3)/3 + \
          (β1+β2+β3)/3*(λ_du1+λ_du2+λ_du3)/3*((T1_hour1+T1_hour2+T1_hour3)/3/2+(MTTR1+MTTR2+MTTR3)/3)
      
    return   val 
      
      
def pfd_RBD_avg_1oo3_dif(λ_du1: float,λ_dd1: float,T1_month1: float,MTTR1: float,β1: float,βd1: float,λ_du2: float,λ_dd2: float,T1_month2: float,MTTR2: float,β2: float,βd2: float,λ_du3: float,λ_dd3: float,T1_month3: float,MTTR3: float,β3: float,βd3: float) -> float:

    λ_d1 = λ_du1 + λ_dd1
    T1_hour1 = T1_month1*730

    λ_d2 = λ_du2 + λ_dd2
    T1_hour2 = T1_month2*730

    λ_d3 = λ_du3 + λ_dd3
    T1_hour3 = T1_month3*730


    Tce1 = (λ_du1/λ_d1)*((T1_hour1/2)+MTTR1)+(λ_dd1/λ_d1)*MTTR1
    Tce2 = (λ_du2/λ_d2)*((T1_hour2/2)+MTTR2)+(λ_dd2/λ_d2)*MTTR2
    Tce3 = (λ_du3/λ_d3)*((T1_hour3/2)+MTTR3)+(λ_dd3/λ_d3)*MTTR3 
   
    Tge1 = (λ_du1/λ_d1)*((T1_hour1/3)+MTTR1)+(λ_dd1/λ_d1)*MTTR1
    Tge2 = (λ_du2/λ_d2)*((T1_hour2/3)+MTTR2)+(λ_dd2/λ_d2)*MTTR2   
    Tge3 = (λ_du3/λ_d3)*((T1_hour3/3)+MTTR3)+(λ_dd3/λ_d3)*MTTR3

    Tg2e1 = (λ_du1/λ_d1)*((T1_hour1/4)+MTTR1)+(λ_dd1/λ_d1)*MTTR1
    Tg2e2 = (λ_du2/λ_d2)*((T1_hour2/4)+MTTR2)+(λ_dd2/λ_d2)*MTTR2   
    Tg2e3 = (λ_du3/λ_d3)*((T1_hour3/4)+MTTR3)+(λ_dd3/λ_d3)*MTTR3
    
    val = (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce1*Tge2*Tg2e3 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce1*Tge3*Tg2e2 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce2*Tge1*Tg2e3 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce2*Tge3*Tg2e1 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce3*Tge1*Tg2e2 + \
          (((1-βd1)*λ_dd1 + (1- β1)*λ_du1)*((1-βd2)*λ_dd2 + (1- β2)*λ_du2)*((1-βd3)*λ_dd3 + (1- β3)*λ_du3))*Tce3*Tge2*Tg2e1 + \
          ((βd1*λ_dd1*MTTR1)+(βd2*λ_dd2*MTTR2)+(βd3*λ_dd3*MTTR3))/3 + \
          (β1*λ_du1*(T1_hour1/2+MTTR1)+β2*λ_du2*(T1_hour2/2+MTTR2)+β3*λ_du3*(T1_hour3/2+MTTR3))/3  
      
    return val
      
      





class device:
    def __init__(self, name: str, lambda_du:float, lambda_dd:float=0, lambda_su:float=0,lambda_sd:float=0,type:str="B", SC:int=1, beta:float=0.05, beta_d:float=0.025, HFT:int=0,MTTR:float=100):
        self._name = name
        if lambda_du > 0.1:
            raise ValueError("λdu/h must be <= 0.1")
        if lambda_dd > 0.1:
            raise ValueError("λdu/h must be <= 0.1")
        if lambda_su > 0.1:
            raise ValueError("λdu/h must be <= 0.1")
        if lambda_sd > 0.1:
            raise ValueError("λdu/h must be <= 0.1")
        if type != "A" and type != "B":
            raise ValueError("Tyepe must be A or B")
        if SC != 1 and SC != 2 and SC != 3 and SC != 4:
            raise ValueError("SC must be 1,2,3 or 4")
        if beta > 0.5:
            raise ValueError("beta must be < 0.5")
        if beta_d > 0.5:
            raise ValueError("beta_d must be < 0.5")
        if HFT != 0 and HFT != 1 and HFT != 2 and HFT != 3 and HFT != 4:
            raise ValueError("HFT must be 0,1,2,3 or 4")
        if MTTR < 0:
            raise ValueError("MTTR must be > 0")
        self._lambda_du = lambda_du
        self._lambda_dd = lambda_dd
        self._lambda_su = lambda_su
        self._lambda_sd = lambda_sd
        self._type = type
        self._SC = SC
        self._beta = beta
        self._beta_d = beta_d
        self._HFT = HFT
        self._MTTR = MTTR
    @property
    def name(self):
        return self._name
    @name.setter
    def name(self, new_name):
        self._name = new_name
    @property
    def lambda_du(self):
        return self._lambda_du
    @property
    def lambda_du_fit(self):
        return self._lambda_du/1e-9    
    @lambda_du.setter
    def lambda_du(self, new_lambda_du):
        if new_lambda_du > 0.1:
            raise ValueError("λdu/h must be <= 0.1")
        else:
            self._lambda_du = new_lambda_du
    def lambda_du_FIT(self, FIT:float):
        if FIT < 0:
            raise ValueError("λdu FIT must be >= 0")
        else:
            self._lambda_du = FIT*1e-9
    @property
    def lambda_dd(self):
        return self._lambda_dd
    @property
    def lambda_dd_fit(self):
        return self._lambda_dd/1e-9    
    @lambda_dd.setter
    def lambda_dd(self, new_lambda_dd:float):
        if new_lambda_dd > 0.1:
            raise ValueError("λdd/h must be <= 0.1")
        else:
            self._lambda_dd = new_lambda_dd
    def lambda_dd_FIT(self, FIT:float):
        if FIT < 0:
            raise ValueError("λdd FIT must be >= 0")
        else:
            self._lambda_dd = FIT*1e-9
    @property
    def lambda_su(self):
        return self._lambda_su
    @property
    def lambda_su_fit(self):
        return self._lambda_su/1e-9    
    @lambda_su.setter
    def lambda_su(self, new_lambda_su:float):
        if new_lambda_su > 0.1:
            raise ValueError("λsu/h must be <= 0.1")
        else:
            self._lambda_su = new_lambda_su
    def lambda_su_FIT(self, FIT:float):
        if FIT < 0:
            raise ValueError("λsu FIT must be >= 0")
        else:
            self._lambda_su = FIT*1e-9
    @property
    def lambda_sd(self):
        return self._lambda_sd
    @property
    def lambda_sd_fit(self):
        return self._lambda_sd/1e-9    
    @lambda_sd.setter
    def lambda_sd(self, new_lambda_sd:float):
        if new_lambda_sd > 0.1:
            raise ValueError("λsd/h must be <= 0.1")
        else:
            self._lambda_sd = new_lambda_sd
    def lambda_sd_FIT(self, FIT:float):
        if FIT < 0:
            raise ValueError("λsd FIT must be >= 0")
        else:
            self._lambda_sd = FIT*1e-9
    @property
    def type(self):
        return self._type  
    @type.setter
    def type(self, new_type:str):
        if new_type != "A" and new_type != "B":
            raise ValueError("Tyepe must be A or B")
        else:
            self._type = new_type
    @property
    def SC(self):
        return self._SC  
    @SC.setter
    def SC(self, new_SC:int):
        if new_SC != 1 and new_SC != 2 and new_SC != 3 and new_SC != 4:
            raise ValueError("SC must be 1,2,3 or 4")
        else:
            self._SC = new_SC
    @property
    def beta(self):
        return self._beta
    @beta.setter
    def beta(self, new_beta:float):
        if new_beta>0.5:
            raise ValueError("Beta must be < 0.5")
        else:
            self._beta = new_beta
    @property
    def beta_d(self):
        return self._beta_d
    @beta_d.setter
    def beta_d(self, new_beta_d:float):
        if new_beta_d>0.5:
            raise ValueError("Beta_d must be < 0.5")
        else:
            self._beta_d = new_beta_d
    @property
    def HFT(self):
        return self._HFT
    @HFT.setter
    def HFT(self, new_HFT:int):
        if new_HFT != 0 and new_HFT != 1 and new_HFT != 2 and new_HFT != 3 and new_HFT != 4:
            raise ValueError("HFT must be 0,1,2,3 or 4")
        else:
            self._HFT = new_HFT
    @property
    def MTTR(self):
        return self._MTTR
    @MTTR.setter
    def MTTR(self, new_MTTR:float):
        if new_MTTR < 0:
            raise ValueError("MTTR must be >0")
        else:
            self._MTTR = new_MTTR
    @property
    def DC_d(self):
        return self._lambda_dd/(self.lambda_d)
    @DC_d.setter
    def DC_d(self, new_DC_d:float):
        if new_DC_d < 0 or new_DC_d > 1:
            raise ValueError("DC must be 0 < DC < 1")
        lambdad= self.lambda_d
        self._lambda_du = lambdad * (1-new_DC_d)
        self._lambda_dd = lambdad * new_DC_d
    @property
    def lambda_s(self):
        return self._lambda_su + self._lambda_sd
    @property
    def lambda_d(self):
        return self._lambda_du + self._lambda_dd
    @property
    def lambda_t(self):
        return self.lambda_d + self.lambda_s
    @property
    def SFF(self):
        return ((self._lambda_dd + self.lambda_s))/(self.lambda_t) #Considering trip if lambda_dd occurs
    @property
    def MTBF_d(self):
        return (1/self.lambda_d)/8760
    @property
    def FS(self):
        return self.lambda_s/self.lambda_t
    def lambda_FIT(self, lambda_du:float, lambda_dd:float, lambda_su:float,lambda_sd:float):
        if lambda_du < 0:
            raise ValueError("λdu FIT must be >= 0 ")
        if lambda_dd < 0:
            raise ValueError("λdd FIT must be >= 0 ")
        if lambda_su < 0:
            raise ValueError("λsu FIT must be >= 0 ")
        if lambda_sd < 0:
            raise ValueError("λsd FIT must be >= 0 ")
        self._lambda_du = lambda_du*1e-9
        self._lambda_dd = lambda_dd*1e-9
        self._lambda_su = lambda_su*1e-9
        self._lambda_sd = lambda_sd*1e-9
    def lambda_hour(self, lambda_du:float, lambda_dd:float, lambda_su:float,lambda_sd:float):
        if lambda_du < 0:
            raise ValueError("λdu hour must be >= 0 ")
        if lambda_dd < 0:
            raise ValueError("λdd hour must be >= 0 ")
        if lambda_su < 0:
            raise ValueError("λsu hour must be >= 0 ")
        if lambda_sd < 0:
            raise ValueError("λsd hour must be >= 0 ")
        self._lambda_du = lambda_du
        self._lambda_dd = lambda_dd
        self._lambda_su = lambda_su
        self._lambda_sd = lambda_sd
    def lambda_year(self, lambda_du:float, lambda_dd:float, lambda_su:float,lambda_sd:float):
        if lambda_du < 0:
            raise ValueError("λdu year must be >= 0 ")
        if lambda_dd < 0:
            raise ValueError("λdd year must be >= 0 ")
        if lambda_su < 0:
            raise ValueError("λsu year must be >= 0 ")
        if lambda_sd < 0:
            raise ValueError("λsd year must be >= 0 ")
        self._lambda_du = lambda_du/8760
        self._lambda_dd = lambda_dd/8760
        self._lambda_su = lambda_su/8760
        self._lambda_sd = lambda_sd/8760
    def header_matrix(self):
        return ["name","λdu","λdd","λsu","λsd","λdu_FIT","λdd_FIT","λsu_FIT","λsd_FIT","type","SC","beta","beta_d","SFF","DC_d","MTBF_d","FS"]
    def val_matrix(self):
        return [self.name, self.lambda_du,self.lambda_dd,self.lambda_su,self.lambda_sd,self.lambda_du_fit,self.lambda_dd_fit,self.lambda_su_fit,
                self.lambda_sd_fit,self.type,self.SC,self.beta,self.beta_d,self.SFF,self.DC_d,self.MTBF_d,self.FS]
class proof_test:
    def __init__(self, name1:str, T1_month:float):
        self.name1 = name1
        self.T1_month = T1_month

class proof_test_1pt:
    def __init__(self, name_pt:str, T_pt_month:float, PDC_pt:float, T1_month:float):
        self.name_pt = name_pt
        self.T_pt_month = T_pt_month
        self.PDC_pt = PDC_pt
        self.T1_month = T1_month

class proof_test_2pt:
    def __init__(self, name_pt1:str, T_pt1_month:float, PDC_pt1:float, name_pt2:str, T_pt2_month:float, PDC_pt2:float,T1_month:float):
        self.name_pt1 = name_pt1
        self.T_pt1_month = T_pt1_month
        self.T_pt2_month = T_pt2_month
        self.PDC_pt1 = PDC_pt1
        self.PDC_pt2 = PDC_pt2
        self.name_pt2 = name_pt2
        self.T1_month = T1_month
        


from PyPFD import PyPFDMarkovTransition

def matrix_mul(A,B):
    resultado = [[0 for _ in range(len(B[0]))] for _ in range(len(A))]

# Multiplicação
    for i in range(len(A)):
        for j in range(len(B[0])):
            for k in range(len(B)):
                resultado[i][j] += A[i][k] * B[k][j]


    return resultado

def markov_cal_2pt(stateM,transitionM,safeM,testsM,testV,maxT):
    
    rM = stateM
    pfd = 0
    
    for i in range(1,maxT):
       rM = matrix_mul(rM,transitionM)

       if i % testV[0] == 0:
           rM = matrix_mul(rM,testsM[0])

       if i % testV[1] == 0:
           rM = matrix_mul(rM,testsM[1])

       if i % testV[2] == 0:
           rM = matrix_mul(rM,testsM[2])    

       for j in range(len(rM[0])):
           if safeM[0][j] == 0:
              pfd+=rM[0][j]

    pfd = pfd/maxT

    #{"transitionM": tmatrix, "testM": testM, "testM_pt1": testM_pt1, "testM_pt2": testM_pt2, "stateVector":stateVector,"safeVector":safeVector}
    return {"stateVector": rM, "pfdavg":pfd}

def pfd_Mkv_avg_1oo1_2pt(λ_du: float,λ_dd: float,λ_s: float,T_pt1_month: float,T_pt2_month: float,T1_month: float,PDC1: float,PDC2: float,MTTR: float):

    T1_hour = T1_month*730
    T_pt1_hour = T_pt1_month*730
    T_pt2_hour = T_pt2_month*730

    markovDictonary = PyPFDMarkovTransition.markovMatrixDict_1oo1_2pt(λ_du, PDC1,PDC2,MTTR,λ_dd,λ_s)
    markovResult = markov_cal_2pt(markovDictonary["stateVector"],markovDictonary["transitionM"],markovDictonary["safeVector"],[markovDictonary["testM_pt1"],markovDictonary["testM_pt2"],markovDictonary["testM"]],[T_pt1_hour,T_pt2_hour,T1_hour],T1_hour)
    return markovResult["pfdavg"]




from PyPFD import PyPFDClass


#                             name,                                 lambda_du,   lambda_dd,   lambda_su,  lambda_sd,    type,   SC,      beta,       beta_d,   HFT,     MTTR)
def sensor_analog_transmitter_generic():
    return PyPFDClass.device("Generic Analog Transmitter",               2e-7,      1.8e-6,        2e-7,     1.8e-6,     "B",    3,      0.04,        0.020,     0,      160)

def sensor_mechanical_switch():
    return PyPFDClass.device("Generic Mechanical Switch",                4e-6,           0,        4e-6,      4e-13,     "A",    2,      0.05,        0.025,     0,      100)

def ls_sil3_certified_plc_generic():
    return PyPFDClass.device("Generic SIL3 Certified PLC",            6.77e-9,           0,     1.85e-7,          0,     "B",    3,      0.05,        0.025,     1,      100)

def fe_valve_onoff_generic():
    return PyPFDClass.device("Generic ON-OFF Failsafe Valve",         3.37e-6,           0,     1.38e-6,          0,     "A",    3,      0.05,        0.025,     0,      100)

def st_IEC61508_Tableb9():
    return PyPFDClass.device("IEC61508-6 Table B9",                    0.5e-5,           0,           0,          0,     "A",    3,      0.10,        0.050,     0,       8)

def sensor_analog_transmitter_proof_test_year_AGAN():
    return PyPFDClass.proof_test("As Good As New Test", 24)



# name_pt:str, T_pt_month:float, PDC_pt:float, T1_month:float):

class proof_test:
    def __init__(self, name1:str, T1_month:float):
        self.name1 = name1
        self.T1_month = T1_month

def st_test():
    return PyPFDClass.proof_test("Std Test", 12)



def sensor_analog_transmitter_proof_test_1pt_handheld():
    return PyPFDClass.proof_test_1pt("HandHeld Simulate Test", 12, 0.54,24)

def st_IEC61508_Tableb0_1pt():
    return PyPFDClass.proof_test_1pt("IEC61508 Table B9 Examplo", 12, 0.90,120)

#name_pt1:str, T_pt1_month:float, PDC_pt1:float, name_pt2:str, T_pt2_month:float, PDC_pt2:float,T1_month:float):

def st_2pt():
    return PyPFDClass.proof_test_2pt("Partial Test1",6,0.3,"Partial test2",12,0.5,24)



from PyPFD import PyPFDRBDAvg
from PyPFD import PyPFDMarkov




def pfd_RBD_avg_KooN_Device(device,proof_test,K,N):
    #(K,N,λ_du,λ_dd,β,βd,T1_month,MTTR)
    return PyPFDRBDAvg.pfd_RBD_avg_KooN(K,N,device.lambda_du,device.lambda_dd, device.beta, device.beta_d,proof_test.T1_month,device.MTTR)

def pfd_RBD_avg_KooN_Device_1pt(device,proof_test_1pt,K,N):
# pfd_avg_KooN_1pt(K,N,λ_du,λ_dd,β,βd,T1_month,T_pt_month,PDC,MTTR)
    return PyPFDRBDAvg.pfd_RBD_avg_KooN_1pt(K,N,device.lambda_du,device.lambda_dd, device.beta, device.beta_d,proof_test_1pt.T1_month,proof_test_1pt.T_pt,proof_test_1pt.DC_pt,device.MTTR)

def pfd_Mkv_avg_1oo1_Device_2pt(device,proof_test_2pt):
#(λ_du: float,λ_dd: float,λ_s: float,T_pt1_month: float,T_pt2_month: float,T1_month: float,PDC1: float,PDC2: float,MTTR: float)
    return PyPFDMarkov.pfd_Mkv_avg_1oo1_2pt(device.lambda_du,device.lambda_dd,device.lambda_s,proof_test_2pt.T_pt1_month,proof_test_2pt.T_pt2_month,proof_test_2pt.T1_month,proof_test_2pt.PDC_pt1,proof_test_2pt.PDC_pt2,device.MTTR)

def pfd_Mkv_avg_1oo1_Device_1pt(device,proof_test_1pt):
#(λ_du: float,λ_dd: float,λ_s: float,T_pt1_month: float,T_pt2_month: float,T1_month: float,PDC1: float,PDC2: float,MTTR: float)
    return PyPFDMarkov.pfd_Mkv_avg_1oo1_2pt(device.lambda_du,device.lambda_dd,device.lambda_s,proof_test_1pt.T_pt_month,proof_test_1pt.T1_month,proof_test_1pt.T1_month,proof_test_1pt.PDC_pt,0,device.MTTR)
